﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Mvc;
using Newtonsoft.Json;

namespace AsyncControllerTest.Controllers
{
    public class ValuesController : ApiController
    {
        /// <summary>
        /// This action does not render any view - APIController just returns data (as XML or JSON). 
        /// Here we return only JSON (see WebApiConfig for configuration that forces JSON formatting)
        /// <returns>JSON formatted Product object</returns>
        public Product Get()
        {
            //return GetProductLongRunningTask();
            var t = LongRunningTaskAsync();
            if(t.IsCompleted) 
              return t.Result;

            return null;
        }

        // simulate long wait with thread.sleep
        private Product GetProductLongRunningTask()
        {          
            Thread.Sleep(5000); // simulate long wait 
            return new Product
            {
                Name = "Soda",
                Price = 2.00M,
                ProductCode = 1
            };
        }

        // now call the same longrunningtask using async await
        private async Task<Product> LongRunningTaskAsync()
        {
            var func = Task<Product>.Factory.StartNew(GetProductLongRunningTask);
            return await func;
        }

        public class Product
        {
            public string Name { get; set; }
            public decimal Price { get; set; }
            [JsonIgnore]
            public int ProductCode { get; set; } // omitted
        }
    }
}